
/*****************************************************************************
**
** File:        mqwrapclose.c
** Purpose:     Natural Interface for MQSeries
** Description: MQCLOSE function
**
** (c) Copyright 2000 by IBM
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>             /* Memory and type conversions */
#include <string.h>             /* strcpy stuff */
#include <ctype.h>              /* is upper stuff */
#include <time.h>               /* time of course */
#include <errno.h>              /* contains errno , last error indicator */
#include <fcntl.h>              /* FILE I/O related protos and defines */
#include <limits.h>		/* FILE I/O related protos and defines */
#include <io.h>             /* defines for file access              */
#include <sys/types.h>          /* for data types sycj as time          */
#include <sys/stat.h>           /* finding out status info              */

#include "cmqc.h"		        /* MQ Headers		*/
#include "natuser.h"	    	/* Natural User Exit	*/
#include "namq.h"               /* MQ interface function prototypes */

static FILE *dbgfile = NULL;

NATFCT NAMQCLOSE NATARGDEF(nparm, parmptr, parmdec)
{
#ifdef MQCLOSE_DEBUG
  static char * func = "NAMQCLOSE : "	;
#endif

  MQHCONN	con_hndl	;
  MQHOBJ        obj_hndl        ;
  MQLONG 	options 	;
  MQLONG 	cc		;
  MQLONG 	rc 		;

#ifdef MQCLOSE_DEBUG
   dbgfile = dbgopen();
#endif

  /*
  ** test number of arguments
  */
  if (nparm != 5 )
       {
#ifdef MQCLOSE_DEBUG
		if (dbgfile != NULL) {
    		fprintf(dbgfile, "\n%s Wrong number of of Parameters: %d" ,func, nparm) ;
		}
		dbgclose();
#endif
		return 1;
  	 }

  /*
  ** move Con Handle from Natural to Local variable
  */

  memmove ((char *) &con_hndl, (char *) parmptr[0], sizeof(MQHCONN));

#ifdef MQCLOSE_DEBUG
  fprintf(dbgfile, "\n%s con handle is '%d'", func , con_hndl ) ;
#endif

  /*
  ** move Object Handle from Natural to Local variable
  */

  memmove ((char *) &obj_hndl, (char *) parmptr[1], sizeof(MQHOBJ));

#ifdef MQCLOSE_DEBUG
	if (dbgfile != NULL) {
  		fprintf(dbgfile, "\n%s Object handle is '%d'", func , obj_hndl ) ;
	}
#endif

  /*
  ** move Options from Natural to Local variable
  */

  memmove ((char *) &options, (char *) parmptr[2],  sizeof(MQLONG));

#ifdef MQCLOSE_DEBUG
  if (dbgfile != NULL) {
  	fprintf(dbgfile, "\n%s options is '%d'", func , options ) ;
  }
#endif

  /*
  ** Call the Close routine
  */

  mq_close ( con_hndl, &obj_hndl, options, &cc, &rc);

#ifdef MQCLOSE_DEBUG
  if (dbgfile != NULL) {
     fprintf(dbgfile, "\n%s handle = %d cc = %d , rc = %d ", func , obj_hndl , cc , rc ) ;
  }
#endif

  /*
  ** move Object Handle to Natural Return block
  */

  memmove ((char *) parmptr[1], (char *) &obj_hndl, sizeof(MQHOBJ));

  /*
  ** move Condition Code to Natural Return block
  */

  memmove ((char *) parmptr[3], (char *) &cc, sizeof(MQLONG));

  /*
  ** move Return Code to Natural Return block
  */

  memmove ((char *) parmptr[4], (char *) &rc, sizeof(MQLONG));

#ifdef MQCLOSE_DEBUG
  if (dbgfile != NULL) {
    dbgclose() ;
  }
#endif

  return 0;

} /* MQCLOSE */


/*
***********************************************************************
			mq_close
***********************************************************************
*/

void mq_close (MQHCONN con_hndl, MQHOBJ * obj_hndl, MQLONG options,
                        MQLONG *cc, MQLONG *rc)
{
#ifdef MQCLOSE_DEBUG
  static char * func = "mq_close : "	;
#endif

    MQCLOSE(con_hndl, obj_hndl, options, cc , rc) ;

#ifdef MQCLOSE_DEBUG
    if ( *cc == MQCC_FAILED) {
  		if (dbgfile != NULL) {
	   		fprintf(dbgfile, "%s Failed to Close %d",func, *rc);
		}
	}
#endif

    return ;

}


