
/*****************************************************************************
**
** File:        mqwrapconnect.c
** Purpose:     Natural Interface for MQSeries
** Description: MQCONN function
**
** (c) Copyright 2000 by IBM
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>             /* Memory and type conversions */
#include <string.h>             /* strcpy stuff */
#include <ctype.h>              /* is upper stuff */
#include <time.h>               /* time of course */
#include <errno.h>              /* contains errno , last error indicator */
#include <fcntl.h>              /* FILE I/O related protos and defines */
#include <limits.h>		/* FILE I/O related protos and defines */
#include <io.h>             /* defines for file access              */
#include <sys/types.h>          /* for data types sycj as time          */
#include <sys/stat.h>           /* finding out status info              */

#include "cmqc.h"		        /* MQ Headers		*/
#include "natuser.h"	    	/* Natural User Exit	*/
#include "namq.h"               /* MQ interface function prototypes */

NATFCT NAMQCONN NATARGDEF(nparm, parmptr, parmdec)
{
#ifdef MQCONN_DEBUG
  static char * func = "NAMQCONN : "	;
#endif

  MQHCONN	con_hndl	;
  MQCHAR48 	con_name	;
  MQLONG 	cc		;
  MQLONG 	rc 		;

  /*
  ** test number of arguments
  */
  if (nparm != 4)
       {
#ifdef MQCONN_DEBUG
    		printf("\n%s Not ENough Parameters!" ,func) ;
#endif
		return 1;
  	 }


  /*
  ** move Con name from Natural to Local variable
  */

  memmove ((char *) con_name, (char *) parmptr[0], sizeof(MQCHAR48));

#ifdef MQCONN_DEBUG
  printf("\n%s con name is '%s'", func , con_name ) ;
#endif

  /*
  ** Call the connect routine
  */

  mq_connect ( &con_hndl, con_name, &cc, &rc);

#ifdef MQCONN_DEBUG
  printf("\n%s handle = %d cc is %d , rc = %d ", func , con_hndl , cc , rc ) ;
#endif

  /*
  ** move Handle to Natural Return block
  */

  memmove ((char *) parmptr[1], (char *) &con_hndl, sizeof(MQHCONN));

  /*
  ** move Condition Code to Natural Return block
  */

  memmove ((char *) parmptr[2], (char *) &cc, sizeof(MQLONG));

  /*
  ** move Return Code to Natural Return block
  */

  memmove ((char *) parmptr[3], (char *) &rc, sizeof(MQLONG));

  return 0;

} /* MQCONN */




/*
***********************************************************************
			mq_connect
***********************************************************************
*/

void mq_connect (MQHCONN * con_hndl, MQCHAR48 con_name, MQLONG *cc, MQLONG *rc)
{
#ifdef MQCONN_DEBUG
  static char * func = "mq_connect : "	;
#endif

    MQCONN(con_name, con_hndl, cc , rc) ;

#ifdef MQCONN_DEBUG
    if ( (*cc == MQCC_WARNING ) && ( *rc == MQRC_ALREADY_CONNECTED) )
	{
	  printf("%s Already connected .. Warning ignored by program",func);
	  *cc = MQCC_OK ;
	}

    if ( *cc == MQCC_FAILED)
	{
	   printf("%s Failed to Connect\n",func);
	}
#endif

    return ;

}


