
/*****************************************************************************
**
** File:        mqwrapput.c
** Purpose:     Natural Interface for MQSeries
** Description: MQPUT function
**
** (c) Copyright 2000 by IBM
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>             /* Memory and type conversions */
#include <string.h>             /* strcpy stuff */
#include <ctype.h>              /* is upper stuff */
#include <time.h>               /* time of course */
#include <errno.h>              /* contains errno , last error indicator */
#include <fcntl.h>              /* FILE I/O related protos and defines */
#include <limits.h>		/* FILE I/O related protos and defines */
#include <io.h>             /* defines for file access              */
#include <sys/types.h>          /* for data types sycj as time          */
#include <sys/stat.h>           /* finding out status info              */

#include "cmqc.h"		        /* MQ Headers		*/
#include "natuser.h"	    	/* Natural User Exit	*/
#include "namq.h"               /* MQ interface function prototypes */

static FILE *dbgfile = NULL;

NATFCT NAMQPUT NATARGDEF(nparm, parmptr, parmdec)
{
#ifdef MQPUT_DEBUG
  static char * func = "NAMQPUT : "	;
	char dbgdata[100];
#endif

  MQHCONN	con_hndl	;
  MQHOBJ        obj_hndl        ;
  MQMD          msg_desc = {MQMD_DEFAULT}       ;
  MQPMO         put_options = {MQPMO_DEFAULT}   ;
  MQLONG 	buf_length 	;
  MQBYTE*       buffer          ;
  MQLONG 	cc		;
  MQLONG 	rc 		;

#ifdef MQPUT_DEBUG
   dbgfile = dbgopen();
#endif

  /*
  ** test number of arguments
  */
  if (nparm != 8 )
       {
#ifdef MQPUT_DEBUG
		if (dbgfile != NULL) {
    		fprintf( dbgfile,"\n%s Wrong number of Parameters! nr: %d" ,func,
                                                        nparm) ;
  			dbgclose();
		}
#endif
	      	return 1;
  	 }

  /*
  ** move Con Handle from Natural to Local variable
  */

  memmove ((char *) &con_hndl, (char *) parmptr[0], sizeof(MQHCONN));

#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
  	fprintf( dbgfile,"\n\n%s con handle: %d", func , con_hndl ) ;
  }
#endif

  /*
  ** move Object Handle from Natural to Local variable
  */

  memmove ((char *) &obj_hndl, (char *) parmptr[1], sizeof(MQHOBJ));

#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
     fprintf( dbgfile,"\n%s object handle: %d", func , obj_hndl ) ;
  }
#endif

  /*
  ** move Message Descriptor from Natural to Local variable
  */

  memmove ((char *) &msg_desc, (char *) parmptr[2], sizeof(MQMD));
#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
	dbgtrunc(dbgdata, msg_desc.StrucId, 4, 4);
  	fprintf( dbgfile,"\n%s msg descriptor: \n  strucid: %s", func , dbgdata ) ;
  	fprintf( dbgfile,"\n  version: %d", msg_desc.Version ) ;
  	fprintf( dbgfile,"\n  report options: %d", msg_desc.Report ) ;
  	fprintf( dbgfile,"\n  message type: %d", msg_desc.MsgType ) ;
  	fprintf( dbgfile,"\n  expiry: %d", msg_desc.Expiry ) ;
  	fprintf( dbgfile,"\n  priority: %d", msg_desc.Priority ) ;
  	fprintf( dbgfile,"\n  orig length: %d \n", msg_desc.OriginalLength) ;
  }
#endif

  /*
  ** move PutOptions from Natural to Local variable
  */

  memmove ((char *) &put_options, (char *) parmptr[3], sizeof(MQPMO));
  if (put_options.RecsPresent > 0)
  {
#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
    fprintf( dbgfile,"\n%s no support for distribution list, RecsPresent = '%d'",
                        func, put_options.RecsPresent);
    dbgclose();
  }
#endif
    return 2;
    }
#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
	dbgtrunc(dbgdata, put_options.StrucId, 4, 4);
  	fprintf( dbgfile,"\n%s Pmo strucid: %s", func , dbgdata ) ;
  	fprintf( dbgfile,"\n%s Pmo version: %d", func , put_options.Version ) ;
  	fprintf( dbgfile,"\n%s Put Options Options: %d", func , put_options.Options ) ;
  	fprintf( dbgfile,"\n%s Put Options Invalid dest count: %d", func ,
                put_options.InvalidDestCount ) ;
  	fprintf( dbgfile,"\n%s Put Options recpresent: %d", func ,
                put_options.RecsPresent ) ;
  	fprintf( dbgfile,"\n%s Put Options resp offset: %d", func ,
                                put_options.ResponseRecOffset ) ;
  	fprintf( dbgfile,"\n%s Put message options size: %d", func, sizeof(MQPMO));
  }
#endif

  /*
  ** move Buffer length from Natural to Local variable
  */

  memmove ((char *) &buf_length, (char *) parmptr[4], sizeof(MQLONG));

#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
  	fprintf( dbgfile,"\n%s buffer length: %d", func , buf_length ) ;
  }
#endif

  /*
  ** Point local variable Buffer to Natural structure
  */
  buffer = (char *) parmptr[5];
  buffer[buf_length] = '\0';

#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
  	fprintf( dbgfile,"\n%s buffer: %s", func , buffer) ;
  }
#endif

  /*
  ** Call the put routine
  */
  mq_put ( con_hndl, obj_hndl, &msg_desc, &put_options, buf_length,
                buffer, &cc, &rc);

#ifdef MQPUT_DEBUG
  if (dbgfile != NULL) {
  	fprintf( dbgfile,"\n%s Put Result: cc is %d , rc = %d \n", func , cc , rc ) ;
  }
#endif

  /*
  ** move Message Descriptor to Natural Return block
  */
  memmove ((char *) parmptr[2], (char *) &msg_desc, sizeof(MQMD));

  /*
  ** move Put options to Natural Return block
  */
  memmove ((char *) &put_options, (char *) parmptr[3], sizeof(MQPMO));

  /*
  ** move Condition Code to Natural Return block
  */

  memmove ((char *) parmptr[6], (char *) &cc, sizeof(MQLONG));

  /*
  ** move Return Code to Natural Return block
  */

  memmove ((char *) parmptr[7], (char *) &rc, sizeof(MQLONG));

#ifdef MQPUT_DEBUG
  dbgclose();
#endif

  return 0;

} /* MQPUT */



/*
***********************************************************************
			mq_put
***********************************************************************
*/
void mq_put (MQHCONN con_hndl, MQHOBJ obj_hndl, MQMD * msg_desc,
               MQPMO * put_options, MQLONG buf_length, MQBYTE * buffer,
               MQLONG *cc, MQLONG *rc)
{
#ifdef MQPUT_DEBUG
  static char * func = "mq_put : "	;
#endif

    MQPUT(con_hndl, obj_hndl, msg_desc, put_options, buf_length,
                        buffer, cc , rc) ;

#ifdef MQPUT_DEBUG
    if ( *cc == MQCC_FAILED)
	{
  		if (dbgfile != NULL) {
		   fprintf( dbgfile,"%s Failed to Put %d",func, *rc);
	 	}
	}
#endif

    return ;

}


